@@ -1,5 +1,5 @@
-(* This file is part of Markup.ml, released under the BSD 2-clause license. See
-   doc/LICENSE for details, or visit https://github.com/aantron/markup.ml. *)
+(* This file is part of Markup.ml, released under the MIT license. See
+   LICENSE.md for details, or visit https://github.com/aantron/markup.ml. *)
 
 open Common
 open Token_tag
@@ -381,7 +381,7 @@ let tokenize report (input, get_location) =
                 match_named (Some (w, m)) (v::(replace @ matched)) [] trie text
               | Trie.Yes m ->
                 let w = Buffer.contents text in
-                finish (Some (w, m)) (v::matched) [])
+                finish (Some (w, m)) (v::(replace @ matched)) [])
         in
         match_named
           None [] [] (Lazy.force named_entity_trie) (Buffer.create 16))
