@@ -1,5 +1,5 @@
-(* This file is part of Markup.ml, released under the BSD 2-clause license. See
-   doc/LICENSE for details, or visit https://github.com/aantron/markup.ml. *)
+(* This file is part of Markup.ml, released under the MIT license. See
+   LICENSE.md for details, or visit https://github.com/aantron/markup.ml. *)
 
 open Common
 
@@ -97,7 +97,10 @@ let tokenize report resolve_reference (input, get_location) =
                   (Printf.sprintf "&#%s%s;" reference_prefix s, "reference",
                    "number out of range")) !throw unresolved
 
-              | Some n -> k (char n)
+              | Some n ->
+                let utf_8_encoded = Buffer.create 8 in
+                add_utf_8 utf_8_encoded n;
+                k (Buffer.contents utf_8_encoded)
               end
 
           | _, c when filter c ->
@@ -232,7 +235,7 @@ let tokenize report resolve_reference (input, get_location) =
           consume_whitespace equals_state)
 
         | l, c ->
-          report_if (not @@ is_name_start_char c) l (fun () ->
+          report_if (not @@ is_name_char c) l (fun () ->
             `Bad_token (char c, "attribute", "invalid name character"))
             !throw (fun () ->
           add_utf_8 name_buffer c;
@@ -379,7 +382,7 @@ let tokenize report resolve_reference (input, get_location) =
     and target_state () =
       next' pi finish_pi begin function
         | _, c when is_whitespace c ->
-          if String.lowercase (Buffer.contents target_buffer) = "xml" then
+          if String.lowercase_ascii (Buffer.contents target_buffer) = "xml" then
             xml_declaration_state ()
           else
             text_state ()
@@ -419,7 +422,7 @@ let tokenize report resolve_reference (input, get_location) =
         report l (`Bad_token ("<?...", pi, "empty")) !throw (fun () ->
         k None)
       else
-        if String.lowercase (Buffer.contents target_buffer) = "xml" then
+        if String.lowercase_ascii (Buffer.contents target_buffer) = "xml" then
           finish_xml ()
         else
           k (Some
@@ -435,7 +438,7 @@ let tokenize report resolve_reference (input, get_location) =
         scan [] l
       in
 
-      let matches s (_, name, _) = String.lowercase name = s in
+      let matches s (_, name, _) = String.lowercase_ascii name = s in
 
       let version_valid s =
         String.length s = 3 &&
@@ -495,7 +498,7 @@ let tokenize report resolve_reference (input, get_location) =
               report l
                 (`Bad_token (value, xml, "must be 'yes' or 'no'")) !throw
                 (fun () ->
-              match String.lowercase value with
+              match String.lowercase_ascii value with
               | "yes" -> k (Some true)
               | "no" -> k (Some false)
               | _ -> k None))
